/*******************************************************************************
* Copyright (C) 2018 Maxim Integrated Products, Inc., All Rights Reserved.
*
* Permission is hereby granted, free of charge, to any person obtaining a
* copy of this software and associated documentation files (the "Software"),
* to deal in the Software without restriction, including without limitation
* the rights to use, copy, modify, merge, publish, distribute, sublicense,
* and/or sell copies of the Software, and to permit persons to whom the
* Software is furnished to do so, subject to the following conditions:
*
* The above copyright notice and this permission notice shall be included
* in all copies or substantial portions of the Software.
*
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
* OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
* MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
* IN NO EVENT SHALL MAXIM INTEGRATED BE LIABLE FOR ANY CLAIM, DAMAGES
* OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
* ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
* OTHER DEALINGS IN THE SOFTWARE.
*
* Except as contained in this notice, the name of Maxim Integrated
* Products, Inc. shall not be used except as stated in the Maxim Integrated
* Products, Inc. Branding Policy.
*
* The mere transfer of this software does not imply any licenses
* of trade secrets, proprietary technology, copyrights, patents,
* trademarks, maskwork rights, or any other form of intellectual
* property whatsoever. Maxim Integrated Products, Inc. retains all
* ownership rights.
*******************************************************************************/

#ifndef MaximInterfaceCore_array_span
#define MaximInterfaceCore_array_span

#include "array.hpp"
#include "span.hpp"
#include "type_traits.hpp"
#include "Unconstructible.hpp"

namespace MaximInterfaceCore {

/// @brief
/// Defines a standard set of types for data fields that are represented as a
/// fixed-size array of elements.
/// @tparam T A const qualifier is not allowed.
template <typename T, size_t N> struct array_span : Unconstructible {
  typedef typename remove_volatile<T>::type element;
  static const size_t size = N;

  typedef MaximInterfaceCore::array<element, N> array;
  typedef MaximInterfaceCore::span<const T, N> const_span;
  typedef MaximInterfaceCore::span<T, N> span;
};

// Specialization for "const T" is not defined.
template <typename T, size_t N> struct array_span<const T, N>;

} // namespace MaximInterfaceCore

#endif
